from .Constants import Constants, Privacy, Expiry, Format
from .Api import SyncApi, AsyncApi
from .Models import User, Paste, PastebinException
from .Client import SyncClient, AsyncClient


__version__ = '0.180331a'
__license__ = 'Bunny'
__author__ = 'BUNNY TMJ'
__author_email__ = 'tmukuroj1337@gmail.com'
__url__ = 'https://github.com/tmjvonboss/PasteBunny.git'

__all__ = [
    # Clients
    'SyncClient', 'AsyncClient',
    # Constants
    'Constants', 'Privacy', 'Expiry', 'Format',
    # Api wrapper
    'SyncApi', 'AsyncApi',
    # Model wrappers
    'User', 'Paste', 'PastebinException',
]



class Constants:

    def __init__(self):
        self.version = "0.1a"
        self.paste_url = "https://pastebin.com/api/api_post.php"
        self.user_url = "https://pastebin.com/api/api_login.php"
        self.raw_paste_url = "https://pastebin.com/api/api_raw.php"
        self.headers = {
            "X-Library": "PasteBunny %s " % self.version,
            "X-Language": "Python 3.6"
        }

    def get_paste_url(self):
        return self.paste_url

    def get_user_url(self):
        return self.user_url

    def get_raw_url(self):
        return self.raw_paste_url

    def get_headers(self):
        return self.headers


class Expiry:

    NEVER = "N"
    MINUTES_10 = "10M"
    HOUR = "1H"
    DAY = "1D"
    WEEK = "1W"
    WEEKS_2 = "2W"
    MONTH = "1M"
    MONTHS_6 = "6M"
    YEAR = "1Y"


class Privacy:

    PUBLIC = "0"
    UNLISTED = "1"
    PRIVATE = "2"


class Format:

    """
    I have a life, I don't plan to do this by myself
    """

    NAN = "text"
    PYTHON = "python"
    PHP = "php"
    PERL = "perl"
    RUBY = "ruby"
    JAVASCRIPT = "javascript"
    JAVA = "java"



from .Api import SyncApi, AsyncApi
from .Models import Paste


class SyncClient:

    def __init__(self, dev_key, username, password):
        self.api = SyncApi(dev_key)
        if username is not None and password is not None:
            self.user_key = self.get_user_key(
                username=username,
                password=password
            )

    def get_user_key(self, username, password):
        return self.api.get_user_key(
            username=username,
            password=password
        )

    def create_paste(self, paste=Paste(), private=False):
        if private is False:
            result = self.api.create_paste(
                code=paste.get_code(),
                name=paste.get_name(),
                expiry=paste.get_expiry(),
                privacy=paste.get_privacy(),
                _format=paste.get_format(),
                user_key=self.user_key
            )
        else:
            result = self.api.create_paste(
                code=paste.get_code(),
                name=paste.get_name(),
                expiry=paste.get_expiry(),
                privacy=paste.get_privacy(),
                _format=paste.get_format(),
            )
        paste.set_url(result)
        paste.set_key(result.split("/")[-1])

    def delete_paste(self, paste_key):
        return self.api.delete_paste(
            paste_key=paste_key,
            user_key=self.user_key
        )

    def get_paste(self, paste_key):
        return self.api.get_paste(
                paste_key=paste_key,
                user_key=self.user_key
        )

    def list_pastes(self, limit=50):
        return self.api.list_pastes(
            user_key=self.user_key,
            limit=limit
        )

    def get_trending_pastes(self):
        return self.api.list_trending_pastes()

    def get_user_data(self):
        return self.api.get_user_data(self.user_key)

    def close(self):
        return self.api.session.close()


class AsyncClient:

    def __init__(self, dev_key, username, password):
        self.api = AsyncApi(dev_key)
        if username is not None and password is not None:
            self.username = username
            self.password = password
        self.user_key = None

    async def login(self):
        self.user_key = await self.get_user_key(
            username=self.username,
            password=self.password
        )

    async def get_user_key(self, username, password):
        return await self.api.get_user_key(
            username=username,
            password=password
        )

    async def create_paste(self, paste=Paste(), private=False):
        if private is False:
            result = await self.api.create_paste(
                code=paste.get_code(),
                name=paste.get_name(),
                expiry=paste.get_expiry(),
                privacy=paste.get_privacy(),
                _format=paste.get_format(),
                user_key=self.user_key
            )
        else:
            result = await self.api.create_paste(
                code=paste.get_code(),
                name=paste.get_name(),
                expiry=paste.get_expiry(),
                privacy=paste.get_privacy(),
                _format=paste.get_format(),
            )
        paste.set_url(result)
        paste.set_key(result.split("/")[-1])

    async def delete_paste(self, paste_key):
        return self.api.delete_paste(
            paste_key=paste_key,
            user_key=self.user_key
        )

    async def get_paste(self, paste_key):
        return self.api.get_paste(
                paste_key=paste_key,
                user_key=self.user_key
        )

    async def list_pastes(self, limit=50):
        return self.api.list_pastes(
            user_key=self.user_key,
            limit=limit
        )

    async def get_trending_pastes(self):
        return self.api.list_trending_pastes()

    async def get_user_data(self):
        return self.api.get_user_data(self.user_key)

    async def close(self):
        return await self.api.session.close()



from .Constants import Constants, Expiry, Privacy
from .Models import PastebinException
import requests
from aiohttp import ClientSession


class SyncApi:

    def __init__(self, dev_key):
        self.dev_key = dev_key
        self.constants = Constants()
        self.session = requests.Session()
        self.session.headers = self.constants.get_headers()

    def do(self, url, parameters):
        with self.session.post(url=url, data=parameters) as r:
            if "Bad API request" in r.text:
                raise PastebinException(r.text.split(", ")[1])
            else:
                return r.text

    def get_user_key(self, username, password):
        return self.do(
            url=self.constants.get_user_url(),
            parameters={
                "api_dev_key": self.dev_key,
                "api_user_name": username,
                "api_user_password": password
            }
        )

    def create_paste(self, code, name, expiry=Expiry.MINUTES_10, privacy=Privacy.PUBLIC, _format='text', user_key=''):
        return self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "paste",
                "api_user_key": user_key,
                "api_paste_privacy": privacy,
                "api_paste_name": name,
                "api_paste_expire_date": expiry,
                "api_paste_format": _format,
                "api_dev_key": self.dev_key,
                "api_paste_code": code
            }
        )

    def delete_paste(self, paste_key, user_key):
        return self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "delete",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
                "api_paste_key": paste_key
            }
        )

    def get_paste(self, paste_key, user_key):
        return self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "show_paste",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
                "api_paste_key": paste_key
            }
        )

    def list_pastes(self, user_key, limit):
        return self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "list",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
                "api_results_limit": limit
            }
        )

    def list_trending_pastes(self):
        return self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "trends",
                "api_dev_key": self.dev_key
            }
        )

    def get_user_data(self, user_key):
        return self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "userdetails",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
            }
        )


class AsyncApi:

    def __init__(self, dev_key):
        self.dev_key = dev_key
        self.constants = Constants()
        self.session = None

    async def do(self, url, parameters):
        if self.session is None:
            self.session = ClientSession(headers=self.constants.get_headers())
        async with await self.session.post(url=url, data=parameters) as r:
            if "Bad API request" in await r.text():
                raise PastebinException(r.text.split(", ")[1])
            else:
                return await r.text()

    async def get_user_key(self, username, password):
        return await self.do(
            url=self.constants.get_user_url(),
            parameters={
                "api_dev_key": self.dev_key,
                "api_user_name": username,
                "api_user_password": password
            }
        )

    async def create_paste(self, code, name, expiry=Expiry.MINUTES_10, privacy=Privacy.PUBLIC, _format='text', user_key=''):
        return await self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "paste",
                "api_user_key": user_key,
                "api_paste_privacy": privacy,
                "api_paste_name": name,
                "api_paste_expire_date": expiry,
                "api_paste_format": _format,
                "api_dev_key": self.dev_key,
                "api_paste_code": code
            }
        )

    async def delete_paste(self, paste_key, user_key):
        return await self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "delete",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
                "api_paste_key": paste_key
            }
        )

    async def get_paste(self, paste_key, user_key):
        return await self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "show_paste",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
                "api_paste_key": paste_key
            }
        )

    async def list_pastes(self, user_key, limit):
        return await self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "list",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
                "api_results_limit": limit
            }
        )

    async def list_trending_pastes(self):
        return await self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "trends",
                "api_dev_key": self.dev_key
            }
        )

    async def get_user_data(self, user_key):
        return await self.do(
            url=self.constants.get_paste_url(),
            parameters={
                "api_option": "userdetails",
                "api_user_key": user_key,
                "api_dev_key": self.dev_key,
            }
        )



class Paste:

    def __init__(self, code=None, name=None, _format=None, expiry=None, privacy=None, size=None, key=None, url=None, hits=None, expire_date=None, date=None):
        self.code = code
        self.name = name
        self.format = _format
        self.expiry = expiry
        self.privacy = privacy
        self.size = size
        self.key = key
        self.url = url
        self.hits = hits
        self.size = size
        self.expire_date = expire_date
        self.date = date
        if size is not None:
            self.size = int(size)
        if hits is not None:
            self.hits = int(hits)

    def set_code(self, text):
        self.code = text

    def set_name(self, name):
        self.name = name

    def set_format(self, _format):
        self.format = _format

    def set_expiry(self, expiry):
        self.expiry = expiry

    def set_privacy(self, privacy):
        self.privacy = privacy

    def set_size(self, size):
        self.size = size

    def set_key(self, key):
        self.key = key

    def set_url(self, url):
        self.url = url

    def set_hits(self, hits):
        self.hits = int(hits)

    def get_code(self):
        return self.code

    def get_name(self):
        return self.name

    def get_format(self):
        return self.format

    def get_expiry(self):
        return self.expiry

    def get_privacy(self):
        return self.privacy

    def get_size(self):
        return self.size

    def get_key(self):
        return self.key

    def get_url(self):
        return self.url

    def get_hits(self):
        return self.hits

    def __repr__(self):
        return "<Paste name=%s>" % self.name


class User:

    def __init__(self, name=None, avatar=None, website=None, email=None, location=None, _type=None):
        self.name = name
        self.avatar = avatar
        self.website = website
        self.email = email
        self.location = location
        if _type == 1:
            self.pro = True
        else:
            self.pro = False

    def get_name(self):
        return self.name

    def get_avatar_url(self):
        return self.avatar

    def get_website(self):
        return self.website

    def get_location(self):
        return self.location

    def get_account_type(self):
        if self.pro is True:
            return "PRO"
        return "REGULAR"

    def __repr__(self):
        return "<User %s [%s]>" % (self.name, self.get_account_type())


class PastebinException(Exception):
    ...



from __future__ import with_statement
import re
import codecs

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

# detect the current version
with open('PasteBunny/__init__.py') as f:
    version = re.search(r'__version__\s*=\s*\'(.+?)\'', f.read()).group(1)
assert version

with codecs.open('README.md', 'r', encoding='utf8') as f:
    setup(
        name='PasteBunny',
        packages=['PasteBunny'],
        version=version,
        description='Pastebin wrapper made by bunnies, for bunnies...',
        long_description=f.read(),
        license='Bunny Licensse',
        author='BUNNY TMJ',
        author_email='tmukkuroj1337@gmail.com',
        url='https://github.com/tmjvonboss/PasteBunny.git',
        keywords=['PasteBunny'],
        classifiers=[
            'Development Status :: 2 - Pre-Alpha',
            'Intended Audience :: Developers',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.1',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: Implementation :: CPython',
            'Programming Language :: Python :: Implementation :: PyPy',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content'
        ],
        install_requires=[
            'requests',
            'aiohttp',
            'aiodns',
            'cchardet',
            'asyncio'
        ],
    )